/*
 * This file is part of smolbote. It's copyrighted by the contributors recorded
 * in the version control history of the file, available from its original
 * location: https://neueland.iserlohn-fortress.net/gitea/aqua/smolbote
 *
 * SPDX-License-Identifier: GPL-3.0
 */

#ifndef SMOLBOTE_BROWSER_H
#define SMOLBOTE_BROWSER_H

#include "session.hpp"
#include "webengine/webprofilemanager.h"
#include <QJsonObject>
#include <QMenu>
#include <QPluginLoader>
#include <QVector>
#include <functional>
#include <memory>
#include <singleapplication.h>

class Configuration;
class BookmarksWidget;
class DownloadsWidget;
class MainWindow;
class Browser final : public SingleApplication
{
    Q_OBJECT

public:
    explicit Browser(int &argc, char *argv[], bool allowSecondary = true);
    ~Browser() final;

    const QVector<MainWindow *> windows() const
    {
        return qAsConst(m_windows);
    }

    BookmarksWidget *bookmarks() const
    {
        return m_bookmarks.get();
    }
    DownloadsWidget *downloads() const
    {
        return m_downloads.get();
    }

signals:
    void pluginAdded(QPluginLoader *);

public slots:
    void about();
    void aboutPlugins();

    void showWidget(QWidget *widget, MainWindow *where) const;
    void open(const QVector<Session::MainWindow> &data, bool merge = true);

    bool loadPlugin(const QString &path);

private:
    struct PluginInfo {
        explicit PluginInfo(QPluginLoader *l)
        {
            this->loader = l;
        }
        ~PluginInfo()
        {
            loader->unload();
            for(auto *m : qAsConst(menus))
                m->deleteLater();
        };

        QPluginLoader *loader;
        QVector<QMenu *> menus;
    };

    Q_DISABLE_COPY(Browser)

    std::shared_ptr<BookmarksWidget> m_bookmarks;
    std::unique_ptr<DownloadsWidget> m_downloads;
    std::unique_ptr<WebProfileManager<false>> m_profileManager{ nullptr };

    QVector<MainWindow *> m_windows;
    QVector<PluginInfo *> m_plugins;
};

#endif // SMOLBOTE_BROWSER_H
